import typing
import logging as logger
from google.cloud import logging
from google.cloud.logging.handlers import CloudLoggingHandler, setup_logging
from google.cloud.logging.resource import Resource


class Logging:
    client = None
    _handler: CloudLoggingHandler = None
    _name: str = None

    def __init__(
            self,
            project: str,
            service_account,
            identifier: str,
            namespace: str = 'default',
            location: str = 'global',
            **kw_args):
        self.client = logging.Client(project, service_account, **kw_args)

        self._name = namespace

        self._handler = CloudLoggingHandler(
            self.client,
            namespace,
            resource=self._build_resource(project, identifier, namespace, location)
        )

    @staticmethod
    def _build_resource(project, identifier, namespace: str = 'default', location: str = 'global'):
        return Resource(
            type='generic_node',
            labels={
                'project_id': project,
                'location': location,
                'namespace': namespace,
                'node_id': identifier
            }
        )

    def get_logger(self, level=logger.INFO, create_context=False):

        if create_context:
            setup_logging(self._handler, log_level=level)
            return

        log_instance = logger.getLogger(self._name)
        log_instance.setLevel(level)
        log_instance.addHandler(self._handler)

        return log_instance
